<?php
/*
	Copyright © Eleanor CMS
	URL: http://eleanor-cms.ru, http://eleanor-cms.com
	E-mail: support@eleanor-cms.ru
	Developing: Alexander Sunvas*
	Interface: Rumin Sergey
	=====
	*Pseudonym. See addons/copyrights/info.txt for more information.
*/
if(!defined('CMS'))
	die('Попытка взлома?');
interface DriverDB
{
	public function Connect(array $params);
	public function Close();
	public function Escape($str);
	public function Query($query);
	public function Error();
	public function ErrorNo();
	public function Transaction($event);
	public function AffectedRows();
	public function FetchRow($qid);
	public function FetchAssoc($qid);
	public function FetchArray($qid);
	public function FetchObject($qid);
	public function NumRows($qid);
	public function NumFields($qid);
	public function RowSeek($int,$qid);
	public function Free($qid);
	public function InsertId();
	public function Version();
}
class Db extends BaseClass
{
	public
		$Driver,
		$prefix,#Префикс Бд
		$count_q;#Счетчик запросов
	protected
		$transaction=0,
		$current_query='';
	/*
		Соединение с БД
		['host'] - сервер БД. Для SQLite - имя файла
		['user'] - пользователь БД
		['pass'] - пароль пользоваетля
		['db'] - база данных
		['prefix'] - префикс таблицы БД
		['force'] - жесткое подключение к БД.
		['type'] - тип БД.
	*/
	public function Connect(array $params=array())
	{
		if(!defined('ELEANOR_DB_DRIVER'))
			define('ELEANOR_DB_DRIVER','mysql');
		$this->count_q=0;
		$this->prefix=isset($params['prefix']) ? $params['prefix'] : $this->Mainclass->sql_tbl_prefix;
		if(!isset($params['host']))
			$params['host']=$this->Mainclass->sql_host;
		if(!isset($params['user']))
			$params['user']=$this->Mainclass->sql_user;
		if(!isset($params['pass']))
			$params['pass']=$this->Mainclass->sql_pass;
		if(!isset($params['db']))
			$params['db']=$this->Mainclass->sql_db;
		if(!isset($params['force']))
			$params['force']=true;
		if(!isset($params['type']))
			$params['type']=ELEANOR_DB_DRIVER;
		$class_name='DbDriver_'.$params['type'];
		if(!class_exists($class_name,false))
			require $this->Mainclass->root_path.'classes/db/db_driver_'.$params['type'].'.php';
		$this->Driver=new $class_name($this);
		if(!$this->Driver->Connect($params))
			$this->Error('connect');
	}

	public function Close()
	{
		$this->current_query='';
		while($this->transaction>0)
			$this->Transaction('COMMIT');
		return $this->Driver->Close();
	}

	public function Version()
	{
		return $this->Driver->Version();
	}

	#Общие функции предназначеные для работы с запросами.
	public function Transaction($event,$sfile='?',$sline='?')
	{
		switch($event)
		{
			case 'BEGIN':
				if($this->transaction>0)
				{
					$this->transaction++;
					return true;
				}
				if($result=$this->Driver->Transaction('BEGIN'))
					$this->transaction=1;
			break;

			case 'COMMIT':
			case 'ROLLBACK':
				if($this->transaction>1)
				{
					$this->transaction--;
					return true;
				}
				elseif($this->transaction==1)
				{
					$result=$this->Driver->Transaction($event);
					$this->transaction--;
				}
				elseif($this->transaction==0)
					return false;
			break;

			default:
				$result=$this->Transaction($event);
			break;
		}
		if($result===false)
			return $this->Error(
								'transaction',
								array(
										'error'=>$this->Driver->Error(),
										'no'=>$this->Driver->ErrorNo(),
										'file'=>$sfile,
										'line'=>$sline,
										'query'=>$event,
								)
			);
		return $result;
	}

	/*
		Обычный запрос. Вместо префикса в нужные места подставляем prefix_.
		Не рекомендуется к использованию т.к. злоумышленник может подставить этот же
		prefix_ в поле запроса и узнать префик таблиц, что не желательно в принципе.
	*/
	public function Query($query='',$sfile='?',$sline='?')
	{
		if(!$query)
			$query=$this->current_query;
		return $this->DoQuery(str_replace('`prefix_','`'.$this->prefix,$query),$sfile,$sline);
	}

	/*
		ForceQuery - более "продвинутый" и безопасный запрос.
		$query - строка с параметрами, вида {"name"}, где name - ключ массива $arr
		$arr - параметры замены. У значений должен быть префикс:
		t_ - название таблицы (вперед допишется префикс) [это значение по-умолчанию]
		i_ - значение должно быть преведено к int виду
		s_ - строка - экраинуется
		!_ - строка берется без изменений
	*/
	public function FQuery($query,$arr=array(),$sfile='?',$sline='?')
	{
		if(!$query)
			$query=$this->current_query;
		foreach($arr as $k=>$v)
		{
			switch(substr($v,0,2))
			{
				case '!_':
					$v=substr($v,2);
				break;
				case 'i_':
				case 's_':
					$v="'".$this->EscapeString(substr($v,2))."'";
				break;
				case 't_':
					$v=substr($v,2);
				default:
					$v='`'.$this->prefix.$v.'`';
			}
			$query=str_replace(array('{"'.$k.'"}',"{'".$k."'}"),$v,$query);
		}
		return $this->DoQuery($query,$sfile,$sline);
	}

	/*
		Функция вставки данных в таблицу
	*/
	public function Insert($table,$arr,$sfile='?',$sline='?',$add='IGNORE')
	{
		$this->DoQuery('INSERT '.$add.' INTO '.($table[0]=='!' ? substr($table,1) : '`'.$this->prefix.$table.'`').$this->GenerateInsert($arr),$sfile,$sline);
		return $this->Insert_id();
	}

	public function InsertUpdate($table,$ins,$upd,$sfile='?',$sline='?',$add='IGNORE')
	{
		$query=($table[0]=='!' ? substr($table,1) : '`'.$this->prefix.$table.'` ').$this->GenerateInsert($ins).' ON DUPLICATE KEY UPDATE ';
		foreach($upd as $k=>&$v)
			$query.=$k[0]=='!' ? '`'.substr($k,1).'`='.str_replace('`prefix_','`'.$this->prefix,$v).',' : '`'.$k.'`='.$this->EscapeString(is_array($v) ? serialize($v) : $v).',';
		$this->DoQuery('INSERT '.$add.' INTO '.rtrim($query,','),$sfile,$sline);
		return $this->Insert_id();
	}

	public function Replace($table,$arr,$sfile='?',$sline='?',$add='')
	{
		$this->DoQuery('REPLACE '.$add.' INTO '.($table[0]=='!' ? substr($table,1) : '`'.$this->prefix.$table.'` ').$this->GenerateInsert($arr),$sfile,$sline);
		return $this->Affected_rows();
	}

	/*
		Служебная функция генерации инсерта.
	*/
	protected function GenerateInsert($arr)
	{
		$keys=$values='';
		$is_arr=null;
		foreach($arr as $k=>&$v)
		{
			if($same=$k[0]=='!')
				$k=substr($k,1);
			$keys.='`'.$k.'`,';
			if($same)
			{
				if(is_array($v))
					foreach($v as &$val)
						$val=str_replace('`prefix_','`'.$this->prefix,$val);
				else
					$v=str_replace('`prefix_','`'.$this->prefix,$v);
			}
			elseif(is_array($v))
				$v=$this->MakeArraySafe($v);
			else
				$v=$this->EscapeString($v,true);
			if($is_arr===null)
			{
				$is_arr=is_array($v);
				$cnt=count($v);
			}
			if($is_arr)
			{
				if($cnt>$t=count($v))
					$v=array_merge($v,array_fill($t,$cnt-$t,'NULL'));
				foreach($v as $k=>&$val)
					$arrs[$k][]=$val;
			}
			else
				$values.=$v.',';
		}
		if($is_arr)
		{
			foreach($arrs as &$v)
				$v=join(',',$v);
			$values='('.join('),(',$arrs).')';
		}
		else
			$values='('.rtrim($values,',').')';
		return '('.rtrim($keys,',').') VALUES '.$values;
	}

	/*
		Функция обновления данных в таблице
	*/
	public function Update($table,$arr,$where='',$sfile='?',$sline='?')
	{
		$query='UPDATE IGNORE '.($table[0]=='!' ? substr($table,1) : '`'.$this->prefix.$table.'`').' SET ';
		foreach($arr as $k=>&$v)
			$query.=$k[0]=='!' ? '`'.substr($k,1).'`='.str_replace('`prefix_','`'.$this->prefix,$v).',' : '`'.$k.'`='.$this->EscapeString(is_array($v) ? serialize($v) : $v,true).',';
		$query=rtrim($query,',');
		if($where)
			$query.=' WHERE '.str_replace('`prefix_','`'.$this->prefix,$where);
		$this->DoQuery($query,$sfile,$sline);
		return $this->Affected_rows();
	}

	/*
		Функция удаления данных из таблицы.
	*/
	public function Delete($table,$where='',$sfile='?',$sline='?')
	{
		if($where)
			$query='DELETE FROM '.($table[0]=='!' ? substr($table,1) : '`'.$this->prefix.$table.'`').' WHERE '.str_replace('`prefix_','`'.$this->prefix,$where);
		else
			$query='TRUNCATE TABLE '.($table[0]=='!' ? substr($table,1) : '`'.$this->prefix.$table.'`');
		$this->DoQuery($query,$sfile,$sline);
		return $this->Affected_rows();
	}

#Дополнительные функции
	public function In($value,$not=false)
	{
		if(is_array($value) and count($value)==1)
			$value=reset($value);
		if(is_array($value))
		{
			$value=$this->MakeArraySafe($value);
			return ($not ? ' NOT' : '').' IN ('.join(',',$value).')';
		}
		return ($not ? '!' : '').'='.$this->EscapeString($value,true);
	}

	/*
		Создаем массив, пригодный для БД. Если нам передается массив в массиве - серилизируем.
	*/
	public function MakeArraySafe($arr)
	{
		foreach($arr as &$v)
			$v=$this->EscapeString(is_array($v) ? serialize($v) : $v,true);
		return $arr;
	}

	public function CurrentQuery()
	{
		return $this->current_query;
	}

	public function SetQuery($v)
	{
		$this->current_query=$v;
	}

	public function SetLimit($offset,$limit=0,$hard=false)
	{
		$offset=(int)$offset;
		$limit=(int)$limit;
		if(preg_match('#LIMIT\s+\d+(,\s*\d+)?$#i',$this->current_query,$m)==0)
		{
			if(!$hard)
				return;
			$this->current_query=substr($this->current_query,0,-strlen($m[0]));
		}
		$this->current_query.=$limit ? ' LIMIT '.$offset.','.$limit : ' LIMIT '.$offset;
	}

	public function ExecQuery()
	{
		return $this->DoQuery($this->current_query);
	}

#Функции для обработки результатов запросов
	public function Affected_rows()
	{
		return $this->Driver->AffectedRows();
	}

	public function Fetch_assoc($qid=-1)
	{
		return $this->Driver->FetchAssoc($qid);
	}

	public function Fetch_row($qid=-1)
	{
		return $this->Driver->FetchRow($qid);
	}

	public function Fetch_array($qid=-1)
	{
		return $this->Driver->FetchArray($qid);
	}

	public function Fetch_object($qid=-1)
	{
		return $this->Driver->FetchObject($qid);
	}

	public function Num_rows($qid=-1)
	{
		return $this->Driver->NumRows($qid);
	}

	public function Num_fields($qid=-1)
	{
		return $this->Driver->NumFields($qid);
	}

	public function Insert_id()
	{
		return $this->Driver->InsertId();
	}

	public function Free($qid=-1)
	{
		return $this->Driver->Free($qid);
	}

	public function RowSeek(int $int,$qid=-1)
	{
		return $this->Driver->RowSeek($int,$qid);
	}

	public function EscapeString($str,$quotes=false)
	{
		if($str===null)
			return 'NULL';
		if(is_array($str))
		{
			foreach($str as &$v)
				$v=$this->EscapeString($v,$quotes);
			return $str;
		}
		if(is_int($str) or is_bool($str) or ctype_digit($str))
			return (int)$str;
		$str=$this->Driver->Escape($str);
		return $quotes ? '\''.$str.'\'' : $str;
	}

#Быстрые запросы

	public function ShowTables()
	{
		return isset($this->Driver->db) ? $this->DoQuery('SHOW TABLES FROM `'.$this->Driver->db.'`',__file__,__line__) : array();
		#Не стоит забывать о хорошей функцие mysql_list_tables и mysql_tablename
	}

	public function OptimizeTable($tname,$add_pref=false)
	{
		if($add_pref)
			if(is_array($tname))
				foreach($tname as &$v)
					$v=$this->prefix.$v;
			else
				$tname=$this->prefix.$tname;
		return $this->DoQuery('OPTIMIZE TABLE '.(is_array($tname) ? '`'.implode('`,`',$tname).'`' : '`'.$tname.'`'),__file__,__line__);
	}

	public function RepairTable($tname,$add_pref=false)
	{
		if($add_pref)
			if(is_array($tname))
				foreach($tname as &$v)
					$v=$this->prefix.$v;
			else
				$tname=$this->prefix.$tname;
		return $this->DoQuery('REPAIR TABLE '.(is_array($tname) ? '`'.implode('`,`',$tname).'`' : '`'.$tname.'`'),__file__,__line__);
	}

#Частные функции - "чистая" работы с БД.
	protected function DoQuery($query,$sfile='?',$sline='?')
	{
		$this->count_q++;
		if($this->Mainclass->debug)
		{
			$debug=array();
			$debug['e']=$query;
			$debug['f']=$sfile;
			$debug['l']=$sline;
			$timer=microtime();
		}
		$this->current_query=$query;
		$qid=$this->Driver->Query($query);
		if($qid===false)
		{
			if($query)
			{
				$error=$this->Driver->Error();
				$errno=$this->Driver->ErrorNo();
			}
			else
			{
				$error='';
				$errno=0;
			}
			$this->Error(
							'query',
							array(
									'error'=>$error,
									'no'=>$errno,
									'file'=>$sfile,
									'line'=>$sline,
									'query'=>$query,
							)
			);
		}
		if($this->Mainclass->debug)
		{
			$debug['t']=round(array_sum(explode(' ',microtime()))-array_sum(explode(' ',$timer)),4);
			$this->Mainclass->debug_arr[]=$debug;
		}
		return $qid;
	}

	public function Error($type,$params=array())
	{
		while($this->transaction>0)
			$this->Transaction('ROLLBACK');
		$lang=$this->Mainclass->LoadLanguage('langs','','db-%');
		$params=array_merge(array('file'=>'?','line'=>'?','query'=>'?','no'=>'?','error'=>'?'),$params);
		switch($type)
		{
			case 'connect':
				$error=$lang['connect'];
			break;
			case 'select_db':
				$error=$lang['select_db'];
			break;
			case 'query':
				if($params['no']==0)
					$error=$lang['empty_query'].$params['file'].'['.$params['line'].'].';
				else
					$error=sprintf(
									$lang['query'],
									'(error #'.$params['no'].') '.htmlspecialchars($params['error']),
									$params['file'].'['.$params['line'].'].'
					);
			break;
			default:
				$error=$params['error'];
			break;
		}
		throw new EE_SQL($error,$params);
	}
}
?>